# GeoSentinel Library
**GeoSentiel** is a set of functions to help download and manipulate sentinel products.
its is currently developed by the **Remote Perception Group** in [**CentroGeo**](geoint.mx).

# Installation
**GeoSentinel** was developed under Debian GNU/Linux, using Bash, Python and other programming languages. 

## Requirements

## Dependencies

# Modules

## APISentinel (`geosentinel/APISentinel.py`)
This class makes easier the usage of [sentinel API](https://github.com/sentinelsat/sentinelsat).
The class constructor requires a username and password from [copernicus project](https://scihub.copernicus.eu/dhus/#/self-registration), more information about the function is available in the function's docstring.
### Dependencies
```bash
sudo pip install sentinelsat
sudo apt-get install libgdal-dev
```
### Download products example
```python
from geosentinel import APISentinel
sentinel = APISentinel('username',  'password')
products = sentinel.getProducts("POLYGON((-89.99450683593753 21.279137394108716,-89.13757324218751 21.2996106049456,-89.30236816406251 20.68418377935238,-90.0494384765625 20.715015145512098,-89.99450683593753 21.279137394108716))", ('20151219', date(2015,12,29)), {"platformname":"Sentinel-2"})
print len(products)
sentinel.downloadProducts(products,".")

```
## WKT Polygon bounding box (`geosentinel/polygonToBox.py`)
Some times you may have a WKT Polygon string that is to long for sentinel API to accept, in this case you can use this script to obtain the bounding box of the polygon and use it with **APISentinel**. This script also has the option to be used with ``gdal_translate`` id the second parameter is set to **False** .

### Dependencies
```bash
sudo apt-get install gdal-bin
pip install GDAL
```

### Example
```python
from geosentinel import APISentinel
from geosentinel import polygonToBox

sentinel = APISentinel('username',  'password')
merida = "POLYGON((-89.99450683593753 21.279137394108716,-89.13757324218751 21.2996106049456,-89.30236816406251 20.68418377935238,-90.0494384765625 20.715015145512098,-89.99450683593753 21.279137394108716))"
products = sentinel.getProducts(polygonToBox.getWKTPolygonBoundingBox(merida, True), ('20150101', '20181024'), {"platformname":"Sentinel-2", "cloudcoverpercentage":"[0 TO 40]"})
sentinel.downloadProducts(products,".")

```

```bash
merida="POLYGON((-89.99450683593753 21.279137394108716,-89.13757324218751 21.2996106049456,-89.30236816406251 20.68418377935238,-90.0494384765625 20.715015145512098,-89.99450683593753 21.279137394108716))"
BOX=$(python3 $HOME/GeoSentinel/geosentinel/polygonToBox.py "$merida")
gdal_translate -projwin $BOX -projwin_srs WGS84 -ot Byte -scale 0 4096 0 255 -of JPEG filein.tiff fileout.jpg
```

## L2A Product generation (`tools/L1CProductListToL2A.sh`)
This script uses [Sen2Cor](http://step.esa.int/main/third-party-plugins-2/sen2cor/) to convert all L1C products in a directory to L2C products. The script takes as an arguments the name of the a directory with a `L1C` Subdirectory where the products are located and the number of jobs to run in parallel.
The new L2A products will be saved in a `L2A` subdirectory


### Dependencies
```bash
sudo apt-get install parallel
wget http://step.esa.int/thirdparties/sen2cor/2.5.5/Sen2Cor-02.05.05-Linux64.run
chmod +x Sen2Cor-02.05.05-Linux64.run
./Sen2Cor-02.05.05-Linux64.run
```

### Example
```bash
L1CProductListToL2A.sh /home/geoint/NAS/sentinelImages/ 12

```

## GRDHL2 Product generation (`tools/GRDHroductListToGRDHL2.sh`)

## Find Local Products(`geosentinel/findProducts.py`)
This script finds sentinel products in a local directory that match a [sentinel API](https://github.com/sentinelsat/sentinelsat) query and and creates symbolic links of this local files to and other directory.

The script works as follows:
1. Using *APISentinel* finds the matching products from a query.
2. Find those products in a local directory (`source`).
3. If one or more products are missing from the local directory, the script prompts the user to download de missing products. 
3. Creates symbolic links (```ln -s```) of this product to an other directory (`destination`)


### Example
```json
{
	"wkt":"POLYGON((-89.62955474853516 21.05230266883862,-89.55093383789061 21.05230266883862,-89.52930450439452 20.965126493194475,-89.6484375 20.873408465645696,-89.72877502441405 20.95294341625548,-89.65805053710938 21.056788298627097,-89.62955474853516 21.05230266883862))" ,
	"platform": "Sentinel-2",
	"productLevel":"L2A",
	"startDate": "20181001",
	"endDate": "20181101",
	"clouds":"40",
	"productsDir":"/home/geoint/NAS/sentinelImages/L2A/",
	"linksDir": "/home/geoint/NAS/merida/",
	"username":"esausername",
	"password":"esapasword"
}


```

```bash
findProducts.py cfg.json

```

## Extract Images from Product ZIP File (`tools/L2AProductListExtractData.sh`)
This script will extract form all the products in a directory the True Colour images (TCI) at 10m and 20m resolution and the Scene Clarification Map at 20m. The files will be saved in subdirectories named with the date of the product.  
### Example
```bash
L2AProductListExtractData.sh /home/geoint/NAS/sentinelImages/ /home/geoint/sentinelImages/outimages/ 12

```

## Merge Images Within a Polygon by Directory (`tool/mergeImagesByDirectory.sh`)
This script uses `gdal_merge.py` and  `gdal_translate` to merge all the images within a directory and to crop the result using the bounding box of the input polygon.
### Example
```bash
mergeImagesByDirectory.sh /home/geoint/NAS/images/ 
```

## Count Total Area for Each L2A SCL Calss (`geosentinel/L2ASCLtoJSON`)
This script count the total number of each of the 12 classes in a SCL image form a Sentinel-2 L2A product.

```json
{
	"wkt":"POLYGON((-89.62955474853516 21.05230266883862,-89.55093383789061 21.05230266883862,-89.52930450439452 20.965126493194475,-89.6484375 20.873408465645696,-89.72877502441405 20.95294341625548,-89.65805053710938 21.056788298627097,-89.62955474853516 21.05230266883862))" ,
	"filename":"/home/geoint/NAS/data/mergecrop/20180214SCL_20m_merged.tif"
}

```bash
L2ASCLtoJSON.py meridaSCL.json > SCLresult.json

```

